#!/usr/bin/python3
""" CherryPy application for file storage"""

# Copyright (C) 2017 Gwyn Ciesla

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from os.path import getsize
import datetime
import getpass
import cherrypy
import sqlalchemy
from sqlalchemy.sql import text
from OpenSSL import crypto

def humansize(inbytes):
    """ Convert numeric bytecount to s string in human-readable format """
    suffix = ['B', 'K', 'M', 'G', 'T']
    count = 0

    if isinstance(inbytes, int):
        while inbytes > 1024:
            count = count + 1
            inbytes = inbytes / 1024

        return str('%.2f' % inbytes) + suffix[count]
    return '0'

def dfree():
    """ Get free bytes for a given path: think df """
    fobj = os.statvfs(os.path.expanduser('~' +  getpass.getuser()))
    space = int(fobj.f_bavail*fobj.f_frsize)

    return space

def dbsize(dbfile):
    """ Get size on disk of db file in bytes """
    return getsize(dbfile)

def objects_size(engine):
    """ Get total size of files stored in database in bytes"""
    with engine.connect() as dbco:
        total = 0
        for size in dbco.execute(text("SELECT SUM(filesize) FROM file_object;")).fetchall():
            total = size[0]

    return total

def usage_stats(database, engine, cert, refresh):
    """ Provide db usage info """
    try:
        rows = "<table align=center border=0 style='border-spacing:20px'>"
        rows = rows + "<tr><td style='font-size:60%'>Free: " \
            + humansize(dfree()) + \
            "</td><td style='font-size:60%'>DB size: " \
            + humansize(dbsize(database)) + \
            "</td><td style='font-size:60%'>DB used by files: " \
            + humansize(objects_size(engine)) + \
            "</td><td style='font-size:60%'>Certificate expires: " \
            + cert_expiry(cert) + \
            "</td><td><form action=" + cherrypy.request.base + "/vacuum method=post>\
            <input type=submit value='Vacuum' style='font-size:60%'></form></td></tr></table>"
    except sqlalchemy.exc.SQLAlchemyError:
        rows = "Database is locked."  + "<meta http-equiv=refresh content=" \
            + refresh + ";url=" + cherrypy.request.base + ">"
        return rows
    return rows

def list_files(engine, refresh):
    """ Display files in DB """
    try:
        with engine.connect() as dbconn:
            rows = "<table align=center border=0 style='border-spacing:20px'>"
            for fileid, stamp, ipaddr, filename, filesize in \
                dbconn.execute(text("SELECT fileid, stamp, ipaddr, filename, filesize FROM file_object ORDER BY stamp DESC;")).fetchall(): # pylint: disable=line-too-long
                rows = rows + "<tr><td align=left style='font-size:80%'>\
                    <form action=" + cherrypy.request.base \
                    + "/download method=post><input \
                    type=hidden name=fileid value=" \
                    + str(fileid) \
                    + "><input type=submit value='v' style='font-size:80%'> " \
                    + filename \
                    + "</form></td><td align=right style='font-size:60%'>"
                rows = rows + '<form action=' + cherrypy.request.base + '/delete method=post>'
                rows = rows + humansize(int(filesize)) + " | " + \
                    datetime.datetime.fromtimestamp(stamp).strftime('%Y-%m-%d %H:%M:%S') \
                    + " | " + ipaddr
                rows = rows + " <input type=hidden name=fileid value=" \
                    + str(fileid) \
                    + "><input type=submit value=X style='font-size:80%'></form></td></tr>"
            rows = rows + "<tr><td colspan=2><hr></td></tr><tr><td align=center colspan=2><form \
                enctype=\"multipart/form-data\" action=" + \
                    cherrypy.request.base + "/upload method=post>"
            rows = rows + "<input type=file \
                name=userfile /><input type=submit value=Upload />\
                    </form></td></tr></table>"
    except sqlalchemy.exc.SQLAlchemyError:
        rows = "Database is locked."  + "<meta http-equiv=refresh content=" \
            + refresh + ";url=" + cherrypy.request.base + ">"
        return rows
    return rows

def cert_expiry(cert):
    """ Get certificate expiration date """
    certfile = open(cert, 'rt').read()
    ctx = crypto
    certobj = ctx.load_certificate(ctx.FILETYPE_PEM, certfile)
    expdate = datetime.datetime.strptime(certobj.get_notAfter().decode("utf-8").replace('Z', ''), \
        '%Y%m%d%H%M%S')
    return str(expdate)
